/*!
 * nextjs-chunk-upload-action
 *
 * Uploading large files with chunking using server action in Next.js
 *
 * [GitHub]: https://github.com/a179346/nextjs-chunk-upload-action
 * [npm]: https://www.npmjs.com/package/nextjs-chunk-upload-action
 * [API Reference]: https://github.com/a179346/nextjs-chunk-upload-action/blob/main/docs/api-reference.md
 */
export type Promisable<T> = T | PromiseLike<T>;
export interface FileLike {
  readonly size: number;
  slice(start?: number, end?: number, contentType?: string): Promisable<Blob>;
}
export type Primitive = string | boolean | number | undefined | null;
export type Metadata = Record<string, Primitive>;
export interface ChunkFormData {
  get(name: 'blob'): Blob;
  get(name: 'offset'): `${number}`;
  get(name: 'length'): `${number}`;
  get(name: 'retry'): `${number}`;
  get(name: 'total'): `${number}`;
  get(name: 'isLastChunk'): 'true' | 'false';
}
export type ChunkUploadHandler<TMetadata extends Metadata = Metadata> = (
  chunkFormData: ChunkFormData,
  metadata: TMetadata
) => Promise<void>;
export interface ChunkUploaderOptions<TMetadata extends Metadata, TFile extends FileLike = File> {
  file: TFile;
  /**
   * The function that defines how the chunk is uploaded to the server.
   */
  onChunkUpload: ChunkUploadHandler<TMetadata>;
  /**
   * The metadata to send with each chunk.
   * This can be used to send additional information like the file name, file type, etc.
   */
  metadata: TMetadata;
  /**
   * The number of bytes to send in each chunk.
   *
   * Default: `5MB` (5 * 1024 * 1024)
   */
  chunkBytes?: number;
  /**
   * Milliseconds to wait before retrying a failed chunk upload.
   * Set to an empty array to disable retries.
   *
   * Default: `[1000, 2000, 4000, 8000]`
   */
  retryDelays?: number[];
  /**
   * A callback that is called when a chunk is uploaded.
   */
  onChunkComplete?: (bytesAccepted: number, bytesTotal: number) => void;
  onSuccess?: () => void;
  onError?: (error: unknown) => void;
  onPaused?: () => void;
  onAborted?: (metadata: TMetadata) => void;
  /**
   * A callback that is called when the status of the uploader changes.
   */
  onStatusChange?: (
    oldStatus: ChunkUploaderStatus | undefined,
    newStatus: ChunkUploaderStatus
  ) => void;
}
export type ChunkUploaderStatus =
  | 'pending'
  | 'uploading'
  | 'pausing'
  | 'paused'
  | 'aborted'
  | 'complete'
  | 'error';
export declare class ChunkUploader<TMetadata extends Metadata, TFile extends FileLike = File> {
  constructor(options: ChunkUploaderOptions<TMetadata, TFile>);
  /*************
   * Public
   *************/
  get status(): ChunkUploaderStatus;
  get bytesUploaded(): number;
  get error(): unknown;
  get file(): TFile;
  /**
   * Start the upload process.
   * returns `false` if the status is not `pending`.
   *
   * status: `pending` -> `uploading` -> `complete` or `error`
   */
  start(): boolean;
  get canStart(): boolean;
  /**
   * Pause the upload process.
   * returns `false` if the status is not `uploading`.
   *
   * Note that the status at the end could be `complete` if the last chunk is being uploaded when the function is called.
   *
   * status: `uploading` -> `pausing` -> `paused` or `complete`
   */
  pause(): boolean;
  get canPause(): boolean;
  /**
   * Resume the upload process.
   * returns `false` if the status is not `paused` or `error`.
   *
   * status: `paused` or `error` -> `uploading` -> `complete` or `error`
   */
  resume(): boolean;
  get canResume(): boolean;
  /**
   * Abort the upload process.
   * returns `false` if the status is not `paused` or `error`.
   *
   * status: `paused` or `error` -> `aborted`
   */
  abort(): boolean;
  get canAbort(): boolean;
  /*************
   * Protected
   *************/
  protected _status: ChunkUploaderStatus;
  protected set status(value: ChunkUploaderStatus);
  protected _position: number;
  protected _error?: unknown;
  protected readonly _file: TFile;
  protected readonly _onChunkUpload: ChunkUploadHandler<TMetadata>;
  protected readonly _chunkBytes: number;
  protected readonly _metadata: Readonly<TMetadata>;
  protected readonly _retryDelays: readonly number[];
  protected readonly _onChunkComplete?: (bytesAccepted: number, bytesTotal: number) => void;
  protected readonly _onSuccess?: () => void;
  protected readonly _onError?: (error: unknown) => void;
  protected readonly _onPaused?: () => void;
  protected readonly _onAborted?: (metadata: TMetadata) => void;
  protected readonly _onStatusChange?: (
    oldStatus: ChunkUploaderStatus | undefined,
    newStatus: ChunkUploaderStatus
  ) => void;
  protected _startUploadFromCurrentPosition(): Promise<void>;
  protected _uploadNextChunk(): Promise<boolean>;
  protected _validateOptions(options: ChunkUploaderOptions<TMetadata, TFile>): void;
  protected _waitForRetry(ms: number): Promise<boolean>;
  protected _statusChangedEventListeners: Partial<Record<ChunkUploaderStatus, Set<() => void>>>;
  protected _addStatusChangedEventListener(status: ChunkUploaderStatus, listener: () => void): void;
  protected _removeStatusChangedEventListener(
    status: ChunkUploaderStatus,
    listener: () => void
  ): void;
}
